﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Linq;
using Windows.Devices.Bluetooth.Advertisement;	// this project specifics, interested in Advertisements Only (no bi-way comms)
using Windows.Storage.Streams;
using Windows.UI.Core;
using System.Threading.Tasks;
using Windows.Foundation;

namespace BeaconExample
{
    class Program
    {
        public static TypedEventHandler<BluetoothLEAdvertisementWatcher, BluetoothLEAdvertisementReceivedEventArgs> OnAdvertisementReceived { get; private set; }
        private class BeaconData
        {
            public String data { get; set; }
            public static BeaconData FromBytes(byte[] bytes)
            {
				// These are specific to the AltBeacon Advertising Data
                if (bytes[0] != 0xBE) { throw new ArgumentException("First byte in array was exptected to be 0xBE", "bytes"); }
                if (bytes[1] != 0xAC) { throw new ArgumentException("Second byte in array was expected to be 0xAC", "bytes"); }
                if (bytes.Length != 24) { throw new ArgumentException("Byte array length was expected to be 24", "bytes"); }

                return new BeaconData {
                    data = BitConverter.ToString(bytes).Replace("-", "")	// removing the dashes in between groups
                };
            }
            public static BeaconData FromBuffer(IBuffer buffer)
            {
                var bytes = new byte[buffer.Length];
                using (var reader = DataReader.FromBuffer(buffer))
                {
                    reader.ReadBytes(bytes);
                }
                return BeaconData.FromBytes(bytes);
            }
        }

        static void Main(string[] args)
        {
            // BLE watcher
            var  watcher = new BluetoothLEAdvertisementWatcher { ScanningMode = BluetoothLEScanningMode.Passive };
            var minSamplingInterval = watcher.MinSamplingInterval;
			
            watcher.SignalStrengthFilter.SamplingInterval = minSamplingInterval;
            watcher.SignalStrengthFilter.OutOfRangeTimeout = TimeSpan.FromMilliseconds(1000);	
            watcher.Start();

            Console.WriteLine("watcher started");
            // while (Console.ReadKey().Key != ConsoleKey.Escape)
            while (true)
            {
                //Console.WriteLine("in the While loop");
            }
            watcher.Stop();
            Console.WriteLine("watcher stopped");
        }

		// here sending filtered data via http server port to be listened to in B4J application
        private static void Watcher_Received(BluetoothLEAdvertisementWatcher watcher, BluetoothLEAdvertisementReceivedEventArgs args)
        {
            var dev = await BluetoothLEDevice.FromBluetoothAddressAsync(args.BluetoothAddress);

            const int PORT_NO = 51042;					// port to sent to
            const string SERVER_IP = "127.0.0.1";		// local machine IP

            foreach (var adv in args.Advertisement.ManufacturerData)	// filter outputs based on adv.CompanyId set below
            {
                //Console.WriteLine("CompanyID:" + adv.CompanyId.ToString());
                const ushort ThatCompanyId = 0x00CD;					// BLE sender device CompanyID as set in there, it is project specific
				
                if (adv.CompanyId == ThatCompanyId) {
                    var beaconData = BeaconData.FromBuffer(adv.Data);
                    Console.WriteLine(beaconData.data);
                    TcpClient tcp_client = new TcpClient(SERVER_IP, PORT_NO);				// open instance
                    NetworkStream nwStream = tcp_client.GetStream();						//
                    byte[] bytesToSend = ASCIIEncoding.ASCII.GetBytes(beaconData.data);		//	
                    nwStream.Write(bytesToSend, 0, bytesToSend.Length);						// write to stream
                    tcp_client.Close();														// close instance 
                }
            }                        
        }
    }
}
